' ================================================================================================
'  Theremino System - ReleDriver V6
' ------------------------------------------------------------------------------------------------
'  - ENG - 
'  This applicazione must be used with "Theremino Switcher V2"
'  further info here: http://www.theremino.com/hardware/outputs/actuators#releswitcherv2
'  Data are sent to a shift register that controls a bank of relays (or other devices)
'  STORE, DATA, CLOCK And 5Volt signals are sent via a Master Module and four shielded wires
'  to receiver modules located at great distance (hundred of meters).
'  If using the "Switcher V2" with the WatchDog circuit please set "WatchDogEnabled" = "True"
' ------------------------------------------------------------------------------------------------
'  - ITA - 
'  Questa applicazione deve essere usata con il "Theremino Switcher V2"
'  maggiori informazioni qui: http://www.theremino.com/hardware/outputs/actuators#releswitcherv2
'  I dati vengono inviati a uno shift register che pilota un banco di relè (o altro) 
'  I segnali STORE, DATA, CLOCK e 5VOLT sono inviati via Master e cavo schermato a quattro fili
'  a moduli ricevitori situati a grande distanza (centinaia di metri).
'  Se si usa lo SwitcherV2 con circuito di WatchDog impostare "WatchDogEnabled" = "True"
' ================================================================================================

Dim NumRele As Int32 = 64 					' Rele number			- Numero di RELE (8,16,24...)
Dim LabelsPerRow As Int32 = 16				' Labels for each row	- Caselle per riga(8,16,24...)
Dim LabelSizeX As Int32 = 40				' Label SIZE X			- Larghezza delle caselle
Dim LabelSizeY As Int32 = 40				' Label SIZE Y			- Altezza delle caselle

Dim SlotStore As Int32 = 1					' Slot ENABLE			- Slot del segnale STORE
Dim SlotData As Int32 = 2					' Slot DATA				- Slot del segnale DATA
Dim SlotClock As Int32 = 3					' Slot CLOCK			- Slot del segnale CLOCK

Dim FinalTimeSeconds As Int32 = 3			' Final time (seconds)	- Tempo finale (secondi)

Dim BitTimeMillisec As Int32 = 4			' Time Each bit (ms)	- Tempo tra i bit (millisec)

Dim WatchDogEnabled As Boolean = False		' Watchdog enabled      - Abilitazione del WatchDog
Dim WatchDogPulsesMillisec As Int32 = 100	' Watchdog pulse len    - Tempo impulsi WatchDog (ms)

Dim ShowDataBits As Boolean = False		    ' Data bits visible     - Visualizza i dati di uscita

Dim ColorUsed As Color = Color.Orange		' Color For RELE USED	- Colore per RELE' USATO
Dim ColorON As Color = Color.Red			' Color For RELE On		- Colore per RELE' ACCESO
Dim ColorArmed As Color = Color.Yellow		' Color For ARMED		- Colore per CASELLA ARMATA
Dim ColorUnused As Color = Color.White		' Color For UNUSED		- Colore per CASELLA NON USATA
Dim ColorBack As Color = Color.LightYellow	' Color For BACK		- Colore per SFONDO FINESTRA 
Dim ColorText As Color = Color.Black		' Color For TEXT		- Colore per TESTO

											' Times for "Progressive sequences" in milliseconds
											' Tempi per le "Sequenze progressive" in millisecondi
Dim ProgressTimesMillisec() As Int32 = {20000, 10000, 5000, 2000, 1000, 800, 600, 400, 200}



' ================================================================================================
'  - ENG - THE SIMPLIFIED MANAGEMENT ENDS HERE - MODIFY THE FOLLOWING CAREFULLY
' ------------------------------------------------------------------------------------------------
'  - ITA - LA GESTIONE SEMPLIFICATA FINISCE QUI - MODIFICARE LA PARTE SEGUENTE CON ATTENZIONE
' ================================================================================================

' ------------------------------------------------------------------------------------------------
'  Main Loop
' ------------------------------------------------------------------------------------------------
Sub Main(ByVal s() As String)
	If Form1 IsNot Nothing Then Return
	Form1 = New form
	InitializeComponents
	MemoryMappedFile_Init
	timeBeginPeriod(1)
	SendEmptyDataString
	StartThread()
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(16)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	SendEmptyDataString
	timeEndPeriod(1)
	StopThread()
End Sub

Sub Button1_GotFocus(sender As Object, e As System.EventArgs) Handles Button1.GotFocus
	Label1.Focus
End Sub

Sub Button1_MouseDown(sender As Object, e As MouseEventArgs) Handles Button1.MouseDown
	If Command <> "" Then Return
	Command = "START"
End Sub

Sub Button2_MouseDown(sender As Object, e As MouseEventArgs) Handles Button2.MouseDown
	If Command <> "" Then Return
	Command = "RESET"
End Sub

Sub Button3_MouseDown(sender As Object, e As MouseEventArgs) Handles Button3.MouseDown
	If Command <> "" Then Return
	Command = "SEQUENCE"
End Sub

Sub TrackBar1_MouseDown(sender As Object, e As EventArgs) Handles TrackBar1.ValueChanged
	Sequence_UpdateWaitTime
	ShowProgressTimes
End Sub

Sub TrackBar2_MouseDown(sender As Object, e As EventArgs) Handles TrackBar2.ValueChanged
	Sequence_UpdateWaitTime
	ShowProgressTimes
End Sub

' ===============================================================
'  Trackbar times (1 to 9) to delays (Milliseconds) conversions
' =============================================================== 

Sub ShowProgressTimes()
	Label1.Text = "Progress times: " & _
				SpeedToString(TrackBar1.value) & _
				" >>> " & SpeedToString(TrackBar2.value)
End Sub

Function SpeedToString(ByVal speed As Int32) As String
	Dim sec As Single = SpeedToDelay(speed) / 1000f
	If sec >= 10 Then
		Return sec.ToString("0", System.Globalization.CultureInfo.InvariantCulture) & " Sec"
	ElseIf sec >= 1 Then
		Return sec.ToString("0.0", System.Globalization.CultureInfo.InvariantCulture) & " Sec"
	Else
		Return sec.ToString("0.00", System.Globalization.CultureInfo.InvariantCulture) & " Sec"
	End If
End Function

Function SpeedToDelay(ByVal speed As Single) As Int32
	Return InterpolatedArray(ProgressTimesMillisec, speed / 10f)
End Function

Function InterpolatedArray(ByVal array() As Int32, ByVal ix As Single) As Int32
	If ix <= 0 Then Return array(0)
	If ix >= array.length - 1 Then Return array(array.length - 1)
	Dim ix_floor As Int32 = CInt(Floor(ix))
	Dim k As Single = ix - ix_floor
	Dim a As Single = array(ix_floor)
	Dim b As Single = array(ix_floor + 1)
	Return CInt(a + k * (b - a))
End Function


' ---------------------------------------------------------------------- User Interface Components
Dim WithEvents Form1 As Form
Dim Label1 As Label = New Label
Dim LabelArray(- 1) As Label
Dim WithEvents Button1 As Button = New Button
Dim WithEvents Button2 As Button = New Button
Dim WithEvents Button3 As Button = New Button
Dim WithEvents TrackBar1 As TrackBar = New TrackBar
Dim WithEvents TrackBar2 As TrackBar = New TrackBar
Dim Label2 As Label = New Label
Dim Label3 As Label = New Label

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Params correction
	NumRele = 8 * (NumRele \ 8) 
	If NumRele < 8 Then NumRele = 8
	LabelsPerRow = 8 * (LabelsPerRow \ 8)
	If LabelsPerRow < 8 Then LabelsPerRow = 8
	' ------------------------------------------------------------------ Labels
	ReDim LabelArray(NumRele - 1)
	InitializeLabels
	Dim x As Int32 = LabelArray(NumRele - 1).right
	Dim y As Int32 = LabelArray(NumRele - 1).bottom
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Rele Driver V6"
	Form1.ClientSize = New System.Drawing.Size(x + 12, y + 120)
	Form1.StartPosition = FormStartPosition.CenterScreen
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = False
	Form1.BackColor = ColorBack
	Form1.KeyPreview = True
	' ------------------------------------------------------------------ Label 1
	Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.ForeColor = ColorText
	Label1.BackColor = ColorUnused 
	Label1.Location = New Point(10, 10)
	Label1.Font = New Font("Arial", 12)
	Label1.Size = New Size(x - 8, 21)
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Button 1
	Button1.BackColor = ColorUnused
	Button1.ForeColor = ColorText
	Button1.Location = New Point(10, y + 8)
	Button1.Font = New Font("Arial", 12)
	Button1.Size = New Size(x - 98, 50)
	Button1.Text = "Start"
	Button1.TabStop = False
	Button1.FlatStyle = FlatStyle.System
	Form1.Controls.Add(Button1)
	' ------------------------------------------------------------------ Button 2
	Button2.BackColor = ColorUnused
	Button2.ForeColor = ColorText
	Button2.Location = New Point(x - 82, y + 8)
	Button2.Font = New Font("Arial", 12)
	Button2.Size = New Size(86, 50)
	Button2.Text = "Reset"
	Button2.TabStop = False
	Button2.FlatStyle = FlatStyle.System
	Form1.Controls.Add(Button2)
	' ------------------------------------------------------------------ Button 3
	Button3.BackColor = ColorUnused
	Button3.ForeColor = ColorText
	Button3.Location = New Point(160, y + 64)
	Button3.Font = New Font("Arial", 12)
	Button3.Size = New Size(x - 308, 50)
	Button3.Text = "Progressive sequence"
	Button3.TabStop = False
	Button3.FlatStyle = FlatStyle.System
	Form1.Controls.Add(Button3)
	' ------------------------------------------------------------------ TrackBar 1
	TrackBar1.Location = New Point(10, y + 64)
	TrackBar1.AutoSize = False
	TrackBar1.Size = New Size(150, 30)
	TrackBar1.Orientation = Orientation.Horizontal
	TrackBar1.TickStyle = TickStyle.TopLeft
	TrackBar1.Maximum = 80
	TrackBar1.Minimum = 0
	TrackBar1.TickFrequency = 10
	TrackBar1.LargeChange = 1
	TrackBar1.Value = 40
	Form1.Controls.Add(TrackBar1)
	' ------------------------------------------------------------------ TrackBar 2
	TrackBar2.Location = New Point(x - 140, y + 64)
	TrackBar2.AutoSize = False
	TrackBar2.Size = New Size(150, 30)
	TrackBar2.Orientation = Orientation.Horizontal
	TrackBar2.TickStyle = TickStyle.TopLeft
	TrackBar2.Maximum = 80
	TrackBar2.Minimum = 0
	TrackBar2.TickFrequency = 10
	TrackBar2.LargeChange = 1
	TrackBar2.Value = 40
	Form1.Controls.Add(TrackBar2)
	' ------------------------------------------------------------------ Label 2
	Label2.BorderStyle = BorderStyle.None
	Label2.ForeColor = ColorText
	Label2.BackColor = color.Transparent 
	Label2.Location = New Point(16, y + 94)
	Label2.Font = New Font("Arial", 11)
	Label2.Size = New Size(160, 21)
	Label2.Text = "Slow                 Fast"
	Form1.Controls.Add(Label2)
	' ------------------------------------------------------------------ Label 3
	Label3.BorderStyle = BorderStyle.None
	Label3.ForeColor = ColorText
	Label3.BackColor = color.Transparent 
	Label3.Location = New Point(x - 132, y + 94)
	Label3.Font = New Font("Arial", 11)
	Label3.Size = New Size(160, 21)
	Label3.Text = "Slow                 Fast"
	Form1.Controls.Add(Label3)
	' ------------------------------------------------------------------ Form Show
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub

Sub InitializeLabels()
	For i As Int32 = 0 To LabelArray.Length - 1
		LabelArray(i) = New Label
		Form1.Controls.Add(LabelArray(i))
		With LabelArray(i)
			.ForeColor = ColorText
			.BackColor = ColorUnused 
			.BorderStyle = BorderStyle.Fixed3D
			.Size = New Size(LabelSizeX, LabelSizeY)
			.Location = New Point(10 + (LabelSizeX + 2) * (i Mod LabelsPerRow), _
								40 + (LabelSizeY + 2) * (i \ LabelsPerRow))
			.Font = New Font("Arial", 12)
			.Text = (i + 1).tostring
			.TextAlign = System.Drawing.ContentAlignment.TopCenter
			Addhandler .Click, AddressOf ClickHandler
		End With
	Next
End Sub

Sub ClickHandler(sender As Object, e As EventArgs)
	If Command <> "" Then Return
	Dim l As Label = CType(sender, Label)
	Dim n As Int32 = CInt(Val(l.Text))
	If l.BackColor <> ColorON And l.BackColor <> ColorUsed Then
		If l.BackColor <> ColorArmed Then
			l.BackColor = ColorArmed
		Else
			l.BackColor = ColorUnused
		End If
	End If
End Sub


' ===============================================================
'  WINDOWS SCHEDULER PRECISION
' =============================================================== 
Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32


' ===============================================================
'  WORKER THREAD
' =============================================================== 
Dim WorkerThread As Threading.Thread
Sub StartThread()
	' --------------------------------------------------------- asynchronously start the Thread 
	WorkerThread = New Threading.Thread(AddressOf ExecuteThread)
	' --------------------------------------------------------- make the thread as background thread.
	WorkerThread.IsBackground = True
	' --------------------------------------------------------- set the Priority of the thread.
	WorkerThread.Priority = Threading.ThreadPriority.highest
	' --------------------------------------------------------- start the thread.
	WorkerThread.Start()
End Sub
Sub StopThread()
	WorkerThread.Abort
End Sub
Sub ExecuteThread
	Do
		Select Case Command
			Case "START"
				ExecCommand_Start()
				Command = ""
			Case "RESET"
				ExecCommand_Reset()
				Command = ""
			Case "SEQUENCE"
				ExecCommand_Sequence()
				Command = ""
			Case Else
				WaitTimeMillisec = 2 * WatchDogPulsesMillisec
				WaitStopWatch.Reset
				WaitStopWatch.Start
				WaitWithWatchDog()
		End Select
		Threading.Thread.Sleep(1)
	Loop Until False
End Sub


' ===============================================================
'  OUTPUT FUNCTIONS
' =============================================================== 
Dim WaitTimeMillisec As Int32
Dim WaitStopWatch As StopWatch = New StopWatch
Sub WaitWithWatchDog()
	While WaitStopWatch.elapsedmilliseconds < WaitTimeMillisec
		If WatchDogEnabled Then
			If (WaitStopWatch.elapsedmilliseconds \ WatchDogPulsesMillisec) Mod 2 = 0 Then
				WriteSlot(SlotStore, 1000)
			Else
				WriteSlot(SlotStore, 0)
			End If
		End If
		Threading.Thread.Sleep(10)
	End While
	WriteSlot(SlotStore, 0)
End Sub

Sub DisableLastRele
	WaitTimeMillisec = FinalTimeSeconds * 1000
	WaitStopWatch.Reset
	WaitStopWatch.Start
	WaitWithWatchDog()
	Dim DataString As String = ""
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			If .BackColor = ColorON Then
				.BackColor = ColorUsed
			End If
		End With
		DataString = DataString & "0"
	Next
	SendDataString(DataString)
End Sub

Sub SendEmptyDataString
	Dim DataString As String = ""
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			.BackColor = ColorUnused
		End With
		DataString = DataString & "0"
	Next
	SendDataString(DataString) 
End Sub

Sub SendDataString(ByVal DataString As String)
	If ShowDataBits Then
		Label1.text = "Data bits: " & DataString
	End If
	Form1.Refresh
	WriteSlot(SlotStore, 0)
	For i As Int32 = DataString.length - 1 To 0 Step -1
		If DataString(i) = "0" Then
			WriteSlot(SlotData, 0)
		Else
			WriteSlot(SlotData, 1000)
		End If
		WriteSlot(SlotClock, 0)
		Threading.Thread.Sleep(BitTimeMillisec)
		WriteSlot(SlotClock, 1000)
		Threading.Thread.Sleep(BitTimeMillisec)
	Next
	WriteSlot(SlotStore, 1000)
	Threading.Thread.Sleep(BitTimeMillisec)
	WriteSlot(SlotData, 0)
	WriteSlot(SlotClock, 0)
	WriteSlot(SlotStore, 0)
End Sub


' ===============================================================
'  COMMANDS EXECUTION
' =============================================================== 
Dim Command As String = ""

Sub DisableControls
	Button1.Enabled = False
	Button2.Enabled = False
	Button3.Enabled = False
End Sub

Sub EnableControls
	Button1.Enabled = True
	Button2.Enabled = True
	Button3.Enabled = True
End Sub

Sub ExecCommand_Reset
	DisableControls
	SendEmptyDataString
	EnableControls
	form1.invalidate
End Sub

Sub ExecCommand_Start
	DisableControls
	Dim DataString As String = ""
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			If .BackColor = ColorON Then
				.BackColor = ColorUsed
			End If
			If .BackColor = ColorArmed Then
				DataString = DataString & "1"
				.BackColor = ColorON
			Else
				DataString = DataString & "0"
			End If 
		End With
	Next
	If DataString.Contains("1") Then
		SendDataString(DataString)
		DisableLastRele
	End If
	EnableControls
	form1.invalidate
End Sub


' ===============================================================
'  SEQUENCE EXECUTION
' =============================================================== 
Dim Sequence_ActualStep As Int32
Dim Sequence_NumSteps As Int32

Sub Sequence_UpdateWaitTime
	WaitTimeMillisec = Sequence_StepTime(Sequence_ActualStep)
End Sub

Function Sequence_StepTime(ByVal StepNumber As Int32) As Int32
	Return SpeedToDelay(trackbar1.value + (stepnumber-1) * (trackbar2.value - trackbar1.value) / (Sequence_NumSteps - 1f))
End Function

Sub ExecCommand_Sequence
	DisableControls
	Sequence_NumSteps = 0
	For i As Int32 = 0 To NumRele -1
		With LabelArray(i)
			If .BackColor = ColorON Then
				.BackColor = ColorUsed
			End If
			If .BackColor = ColorArmed Then
				Sequence_NumSteps += 1
			End If
		End With
	Next
	If Sequence_NumSteps > 0 Then
		For Sequence_ActualStep = 1 To Sequence_NumSteps
			Dim DataString As String = ""
			For i As Int32 = 0 To NumRele -1
				With LabelArray(i)
					If .BackColor = ColorON Then
						.BackColor = ColorUsed
					End If
					If .BackColor = ColorArmed Then
						DataString = DataString & "1"
						.BackColor = ColorON
						Exit For
					Else
						DataString = DataString & "0"
					End If
				End With
			Next
			DataString = DataString & StrDup(NumRele - DataString.Length, "0")
			WaitStopWatch.Reset
			WaitStopWatch.Start
			SendDataString(DataString)
			If Sequence_ActualStep < Sequence_NumSteps Then
				Sequence_UpdateWaitTime
				Button3.text = "Progressive sequence" & vbcrlf & "Waiting " & WaitTimeMillisec.ToString & " ms"
				WaitWithWatchDog()
			End If
		Next
		Button3.text = "Progressive sequence"
		DisableLastRele
		End If
	EnableControls
	form1.Invalidate
End Sub






