' ================================================================================================
'  THEREMINO SERIAL BRIDGE
' ================================================================================================
'  This script receives from Serial-Line the data coming from a Geiger-Sensor
'  and transfers them to a Theremino-Slot
' ------------------------------------------------------------------------------------------------
'  Questo script riceve i dati provenienti sulla linea seriale da un Sensore-Geiger
'  e li trasferisce su uno Slot del sistema Theremino
' ================================================================================================

' --------------------------------------- COM-PORT number and SLOT number
Const PortNumber As Short = 7
Const SlotNumber As Short = 0

Sub Main(ByVal s() As String)
	if Form1 IsNot Nothing then return
	Form1 = New Form
	InitializeComponents
	MemoryMappedFile_Init
	Connect_ComPort()
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	Disconnect_ComPort()
End Sub

Private WithEvents comPort As New System.IO.Ports.SerialPort()

Private Sub Connect_ComPort()
	' -------------------------------------------- If the port is Open then close it
	Disconnect_ComPort()
	' -------------------------------------------- Set PORT and SPEED
	comPort.PortName = "COM" & PortNumber.ToString
	comPort.BaudRate = 9600
	comPort.Parity = System.IO.Ports.Parity.None
	comPort.DataBits = 8
	comPort.StopBits = System.IO.Ports.StopBits.One
	' -------------------------------------------- Set port params
	'comPort.InputLen = 5 '(no equivalent in VbNet)
	comPort.ReceivedBytesThreshold = 1
	'comPort.ReadBufferSize = 6 '(default = 4096)
	' -------------------------------------------- Disable DTR
	comPort.DtrEnable = False
	' -------------------------------------------- Try to open the port
	Try
		comPort.Open()
		Label1.Text = "COM-Port " & PortNumber.ToString & " is opened"
		Label1.Text &= " - Transferring serial data to Slot " & SlotNumber.ToString 
	Catch
		Label1.Text = "ERROR - Can not open COM-Port " & PortNumber.ToString
	End Try
End Sub

Private Sub Disconnect_ComPort()
	If comPort.IsOpen Then
		comPort.Close()
	End If
End Sub

' -----------------------------------------------------------------------------------------
' Riceivin from serial port 10 times per second the pulse sum counted by the pic firmware
' When reaching the count 65536 (16 bit) the pic resets to zero.
' So the received number is alway from 0 to 65535
' -----------------------------------------------------------------------------------------
Private Sub comPort_DataReceived(ByVal sender As Object, _
								ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) _
								Handles comPort.DataReceived
								
	Dim str As String = comPort.ReadExisting
	str = str.replace(VbCr, "|")
	str = str.replace(VbLf, "|")
	'str = str.replace(" ", "|")
	dim s as single = CSng(Val(str))
	WriteSlot(SlotNumber, s)
	Label2.Text = "Serial data received = " & str & "  Len = " & len(str).ToString & "  Value = " & s.ToString
	'comPort.DiscardInBuffer
End Sub


' ---------------------------------------------------------------------- User Interface Components
WithEvents Form1 As Form
Dim Label1 As Label = new Label
Dim Label2 As Label = new Label

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Bridge V1.0 - Bridge from serial line to Theremino-Slot" 
	Form1.ClientSize = New System.Drawing.Size(540, 70)
	Form1.MinimumSize = Form1.Size
	Form1.FormBorderStyle = FormBorderStyle.FixedSingle
	Form1.MaximizeBox = False
	Form1.StartPosition = FormStartPosition.CenterScreen
	' ------------------------------------------------------------------ Label 1
	Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.Location = New Point(10, 10)
	Label1.Font = New Font("Arial", 12 )
	Label1.Size = New Size(Form1.ClientSize.Width - 20, 21)
	Label1.Anchor = AnchorStyles.top or AnchorStyles.Left or AnchorStyles.right
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Label 2
	Label2.BorderStyle = BorderStyle.Fixed3D
	Label2.Location = New Point(10, 40)
	Label2.Font = New Font("Arial", 12 )
	Label2.Size = New Size(Form1.ClientSize.Width - 20, 21)
	Label2.Anchor = AnchorStyles.top or AnchorStyles.Left or AnchorStyles.right
	Form1.Controls.Add(Label2)	
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.show
End Sub

